/*********************************************************************
 *       This file is part of the Mandala library source code.       *
 *                   Copyright (C) 2002                              *
 *                                                                   *
 *       Authors : eipi - eipiequalsnoone@users.sf.net               *
 *                                                                   *
 * This library is free software; you can redistribute it and/or     *
 * modify it under the terms of the GNU Lesser General Public        *
 * License as published by the Free Software Foundation; either      *
 * version 2.1 of the License, or (at your option) any later version.*
 *                                                                   *
 * This library is distributed in the hope that it will be useful,   *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU *
 * Lesser General Public License for more details.                   *
 *                                                                   *
 * You should have received a copy of the GNU Lesser General Public  *
 * License along with this library; if not, write to                 *
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,   *
 * Boston, MA  02111-1307  USA                                       *
 *********************************************************************/
package agents;

import mandala.jacob.*;

/** 
    <p>Generic abstract Agent class.</p>

    <p>This class should be overriden to be able to do something.</p>

*/

public abstract class Agent implements Runnable, Serializable {
    private transient AgentServer hostedServer; // local

    Agent(final AgentServer hostedServer) {
        this.hostedServer = hostedServer;
    }

    public abstract void init(final AgentServer hostedServer);

    /** 
        <p>Send this Agent to the specified <code>destination</code>
        AgentServer.</p>
    */
    public final void migrate(final AgentServer destination) 
        throws MigrationException {
        
        onMigrating(destination);
        final Agent agent = hostedServer.activeMap.remove(id);
        assert this == agent;
        
        ARFactory factory = new SORFactory(destination.activeMap);
        jaya.agents.Agent proxy = factory.getInstance(agent);
            
    }

    public void onMigrating(AgentServerProxy destination){
        System.err.println("Agent.onMigrating()#begin");
        ;
        System.err.println("Agent.onMigrating()#end");
    }
    
    public void onMigration(AgentServerProxy destination){
        System.err.println("Agent.onMigration()#begin");
        ;
        System.err.println("Agent.onMigration()#end");
    }

    public void handleMessage(Object type){
        System.err.println("Agent.handleMessage()#begin");
        ;
        System.err.println("Agent.handleMessage()#end");
    }

    public void run(){
        System.err.println("Agent.run()#begin");
        ;
        System.err.println("Agent.run()#end");
    }
    
        
    public String toString(){
        return Agent.getClass().getName() + 
            "[hostedServer: " + hostedServer + 
            "]";
    }


    /**********************************************************/
    /********************* Methods for AgentServer ************/
    /**********************************************************/
    
    final void migrated(final AgentServer newCurrent) {
        final AgentServer source = this.currentServer;
        this.currentServer = newCurrent;
        onMigration(source.getProxy(), newCurrent);
        new Thread(this).start();
    }
}
