/*********************************************************************
 *       This file is part of the Mandala library source code.       *
 *                   Copyright (C) 2002                              *
 *                                                                   *
 *       Authors : eipi - eipiequalsnoone@users.sf.net               *
 *                                                                   *
 * This library is free software; you can redistribute it and/or     *
 * modify it under the terms of the GNU Lesser General Public        *
 * License as published by the Free Software Foundation; either      *
 * version 2.1 of the License, or (at your option) any later version.*
 *                                                                   *
 * This library is distributed in the hope that it will be useful,   *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU *
 * Lesser General Public License for more details.                   *
 *                                                                   *
 * You should have received a copy of the GNU Lesser General Public  *
 * License along with this library; if not, write to                 *
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,   *
 * Boston, MA  02111-1307  USA                                       *
 *********************************************************************/
package agents;

import mandala.jacob.*;

/** 
    <p>Agent proxy class.</p>
*/

public final class AgentProxy implements Runnable, Serializable {

    private jaya.agents.Agent proxy;


    public void post(Object message) {
        proxy.handleMessage(message);
    }
    
    public static AgentProxy createAgent(Class agentClass, 
                                         AgentServer server) {
        
        
        server.createAgent(agentClass, this);
    }

    public long hashCode() {
        return id;
    }

    public boolean equals(Object o) {
        if (o == this) return true;
        
        if (o instanceof AgentProxy) {
            proxy = (AgentProxy) o;
            
            return proxy.id == id;
        }

        return false;
    }
    
    /** 
        <p>Return the AgentServer which host the Agent this AgentProxy is
        refering to.</p>
    */
    public final String getHostedServer(){
        return hostedServer;
    }
    
    public String toString(){
        return AgentProxy.getClass().getName() + 
            "[hostedServer: " + hostedServer + 
            "]";
    }


}
