package pi;

import mandala.rami.*;
import mandala.jacob.*;

public class ARP_AR_ARP_RSOR_AR_PiComputer {
    public static void main(String[] args) {
        if (args.length != 3) {
            System.err.println(PiComputer
                               .usage(ARP_AR_ARP_RSOR_AR_PiComputer.class, 
                                      "RemoteActiveMapAddress"));
            System.exit(1);
        }
                
        long startChrono = System.currentTimeMillis();

        IntInterval interval = new IntInterval(0,
                                               Integer.parseInt(args[0]));
                
        // gets a proxy on a remote active map
        ActiveMap activeMap = null;
        try{
            activeMap = (mandala.jacob.remote.RemoteActiveMap)
                new javax.naming.InitialContext().lookup(args[2]);
        }catch(Exception e) {
            e.printStackTrace();
            System.exit(1);
        }
        

        
        // Constructs the factory which generates
        // ARP(AR(ARP(RSOR(AR(computer))))) 

        // The headerFactory generates AR
        Framework.Factory headerFactory = new mandala.rami.impl.ARFactory();
        
        
        // The tailerFactory generates ARP(RSOR(AR()))
        
        Framework.Factory innerHeaderFactory = new SORFactory(activeMap);
        // The inner tailerFactory generates AR, but futures must be remote
        // compliant.
        mandala.rami.impl.FutureFactory futureFactory = 
            ((mandala.jacob.remote.RemoteActiveMap) activeMap)
            .getDefaultFutureFactoryWrapper(mandala.rami.impl.ARFactory
                                            .getDefaultFutureFactory());
        Framework.Factory innerTailerFactory =
            new mandala.rami.impl.ARFactory(futureFactory);
        
        // Then this factory generates ARP(RSOR(AR())) it is the tailer !
        Framework.Factory tailerFactory = 
            new mandala.rami.impl.ARPFactory(innerHeaderFactory, 
                                             innerTailerFactory);

        // Then this factory generates ARP(AR(ARP(RSOR(AR()))))
        Framework.setFactory(new mandala.rami.impl.ARPFactory(headerFactory,
                                                              tailerFactory));
        
        jaya.pi.PiComputer computer = new jaya.pi.PiComputer(interval.end);
        
        // Setting callbacks.
        mandala.rami.impl.BenchmarkerCallback benchmarker = 
            new mandala.rami.impl.BenchmarkerCallback();
        mandala.jacob.CleanerCallback cleaner = 
            new mandala.jacob.CleanerCallback();
        mandala.rami.Callback sorCallback = 
            new mandala.rami.impl.CallbackList(new Callback[]{benchmarker,
                                                              cleaner});

        mandala.rami.AsynchronousReferencePair arp = 
            (mandala.rami.AsynchronousReferencePair) 
            computer.getAsynchronousReference();
        mandala.rami.AsynchronousReferencePair innerARP = 
            (AsynchronousReferencePair) arp.getTail();
        
        arp.getCallbackManager().setGlobalCallback(benchmarker);
        arp.getHead().getCallbackManager().setGlobalCallback(benchmarker);
        innerARP.getCallbackManager().setGlobalCallback(benchmarker);
        innerARP.getHead().getCallbackManager().setGlobalCallback(sorCallback);
        innerARP.getTail().getCallbackManager().setGlobalCallback(benchmarker);

        long callStart = System.currentTimeMillis();
        benchmarker.startBenchmark(callStart);

        
        PiComputer.Result result = null;
        try{
            result = (PiComputer.Result)
                computer.rami_compute(interval, args[1]).waitForResult();
        }catch(Throwable t) {
            t.printStackTrace();
            System.exit(1);
        }
        
        PiComputer.logResult(interval,
                             1,
                             result,
                             System.currentTimeMillis() - startChrono);
    }
}
   
        
