package pi;


import mandala.rami.*;
import mandala.jacob.*;

public class ARP_AR_RSOR_PiComputer {
    public static void main(String[] args) {
        if (args.length != 3) {
            System.err.println(PiComputer.usage(ARP_AR_RSOR_PiComputer.class, 
                                                "RemoteActiveMapAddress"));
            System.exit(1);
        }
                
        long startChrono = System.currentTimeMillis();

        IntInterval interval = new IntInterval(0, Integer.parseInt(args[0]));
        

        // gets a proxy on a remote active map
        mandala.jacob.remote.RemoteActiveMap activeMap = null;
        try{
            activeMap = (mandala.jacob.remote.RemoteActiveMap)
                java.rmi.Naming.lookup(args[2]);
        }catch(Exception e) {
            e.printStackTrace();
            System.exit(1);
        }
                
        // Constructs the factory which generates ARP(AR(RSOR(computer)))
        
        // The headerFactory generates AR
        Framework.Factory headerFactory = new mandala.rami.impl.ARFactory();
        
        // The tailerFactory generates (remote) SOR. It uses the
        // DefaultFutureFactoryWrapper
        mandala.rami.impl.FutureFactory futureFactory = 
             activeMap
            .getDefaultFutureFactoryWrapper(SORFactory
                                            .getDefaultFutureFactory());
        Framework.Factory tailerFactory = new SORFactory(activeMap,
                                                         futureFactory);
        
        // Then this factory generates ARP(AR(RSOR()))
        Framework.setFactory(new mandala.rami.impl.ARPFactory(headerFactory,
                                                              tailerFactory));
        jaya.pi.PiComputer computer = new jaya.pi.PiComputer(interval.end);
        
        // Setting callbacks.
        mandala.rami.impl.BenchmarkerCallback benchmarker = 
            new mandala.rami.impl.BenchmarkerCallback();
        mandala.jacob.CleanerCallback cleaner = 
            new mandala.jacob.CleanerCallback();
        mandala.rami.Callback sorCallback = 
            new mandala.rami.impl.CallbackList(new Callback[]{benchmarker,
                                                              cleaner});
        
        mandala.rami.AsynchronousReferencePair arp = 
            (mandala.rami.AsynchronousReferencePair) 
            computer.getAsynchronousReference();
        arp.getCallbackManager().setGlobalCallback(benchmarker);
        arp.getHead().getCallbackManager().setGlobalCallback(benchmarker);
        arp.getTail().getCallbackManager().setGlobalCallback(sorCallback);
         
        long callStart = System.currentTimeMillis();
        benchmarker.startBenchmark(callStart);
        
        PiComputer.Result result = null;
        try{
            result = (PiComputer.Result)
                computer.rami_compute(interval, args[1]).waitForResult();
        }catch(Throwable t) {
            t.printStackTrace();
            System.exit(1);
        }
        
        PiComputer.logResult(interval,
                             1,
                             result,
                             System.currentTimeMillis() - startChrono);
    }
}
   
        
