package pi;

import mandala.util.*;
import mandala.rami.*;
import mandala.rami.impl.*;

public class TestPerfPiComputer   {
    public static Syslog syslog = PiComputer.syslog;

    public static void main(String[] args) {
        if (args.length != 3) {
            syslog.error(PiComputer.usage(TestPerfPiComputer.class, 
                                          "subsumsNumber"));
            System.exit(1);
        }
        
        long startChrono = System.currentTimeMillis();

        int scale = Integer.parseInt(args[0]);
        IntInterval[] intervals = new IntInterval[Integer.parseInt(args[2])];
        PiComputer.distribute(intervals, scale);
        
        // Dummy section to prevent caches effects!
        PiComputer computer = new PiComputer(100);
        jaya.pi.PiComputer proxy;
        
        // Disabling logging in piComputer
        int dummyCallsNb = 1000;
        syslog.log("** " + dummyCallsNb + 
                   " dummy direct calls to " + 
                   "prevent caches effects... wait **\n");
        
        PiComputer.syslog.LOG = false;
        IntInterval dummy = new IntInterval(10, 20);
        for (int i = 0; i < dummyCallsNb; i++) {
            computer.compute(dummy, args[1]);
        }
                
        PiComputer.syslog.LOG = true;
        syslog.log("** " + dummyCallsNb + 
                   " dummy indirect calls to " + 
                   "prevent caches effects... wait **\n");
        PiComputer.syslog.LOG = false;
        for (int i = 0; i < dummyCallsNb; i++) {
            proxy = (jaya.pi.PiComputer) 
                Framework.getSemiTransparentAsynchronousProxy(computer);
            
            try{
                proxy.rami_compute(dummy, args[1]).waitForResult();
            }catch(Throwable t) {
                syslog.error(t);
                System.exit(1);
            }
        }
        
        dummy = null;
        PiComputer.syslog.LOG = true;
	
	Framework.Factory arf = null;
	// **************** Synchonous **************** 
        syslog.log("-- Using synchronous policy --\n");
	// We must instantiate a new PiComputer each time since
        // Framework implements a singleton mechanism !!
	arf = new ARFactory(new AsynchronousPolicyFactory() {
		public AsynchronousPolicy getAsynchronousPolicy(Object o) {
		    return new SynchronousSemantic();
		}
	    });
        proxy = new jaya.pi.PiComputer(scale, arf);
	System.gc();
	compute(proxy, intervals, args[1]);
        
        // **************** Threaded ****************
        syslog.log("-- Using threaded policy --\n");
        arf = new ARFactory(new ThreadedAPFactory());
        proxy = new jaya.pi.PiComputer(scale, arf);
	System.gc();
        compute(proxy, intervals, args[1]);
        
        // **************** ThreadPooled ****************
        syslog.log("-- Using thread-pooled policy --\n");
        arf = new ARFactory(new ThreadPooledAPFactory());
        proxy = new jaya.pi.PiComputer(scale, arf);
        ThreadPool threadPool = ((ThreadPooledPolicy) 
				 ((AsynchronousReferenceImpl) 
				  proxy
				  .getAsynchronousReference())
				 .getAsynchronousPolicy()).getThreadPool();
	
        threadPool.getSyslog().DEBUG = false;
	System.gc();
        compute(proxy, intervals, args[1]);
        
        // **************** Fifo ****************
        syslog.log("-- Using FIFO policy --\n");
        arf = new ARFactory(new FifoAPFactory());
        proxy = new jaya.pi.PiComputer(scale, arf);
	System.gc();
        compute(proxy, intervals, args[1]);

        // **************** Random ****************
        syslog.log("-- Using Random policy --\n");
        arf = new ARFactory(new RandomAPFactory());
        proxy = new jaya.pi.PiComputer(scale, arf); 
        System.gc();
        compute(proxy, intervals, args[1]);

        // **************** Pair ****************
        syslog.log("-- Using AsynchronousReferencePair --\n");
	ARFactory arf2 = new ARFactory(new ThreadPooledAPFactory());
        arf = new ARPFactory(arf2, arf);
        proxy = new jaya.pi.PiComputer(scale, arf);
	System.gc();
        compute(proxy, intervals, args[1]);

        
        // **************** Direct ****************
        syslog.log("-- Using direct PiComputer --\n");
        IntInterval interval = new IntInterval(0, scale);
        computer = new PiComputer(scale);
        long localStartChrono = System.currentTimeMillis();
	System.gc();
        PiComputer.logResult(interval, 
                             1,
                             computer.compute(interval, args[1]),
                             (System.currentTimeMillis() - localStartChrono));
        
        syslog.log("Everything done in " + 
                   (System.currentTimeMillis() - startChrono) + "ms");
    }
    
    public static void compute(jaya.pi.PiComputer proxy, 
                               IntInterval[] intervals,
                               String strategy) {
        System.gc();
                        
        FutureClient[] futures = new FutureClient[intervals.length];
        long[] localstartChronos = new long[intervals.length];
        long[] requestTime = new long[intervals.length];
        
        StringBuffer sb = new StringBuffer(500);
                
        long startChrono = System.currentTimeMillis();
                
        for (int i = 0; i < intervals.length; i++) {
            localstartChronos[i] = System.currentTimeMillis();
            // Asynchronous call
            futures[i] = proxy.rami_compute(intervals[i], strategy);
            
            requestTime[i] = System.currentTimeMillis() - localstartChronos[i];
        }
                
        // Computation can be done here 
        long averageTime = 0;
        sb.append("Request time summary \n");
        for (int i = 0; i < intervals.length; i++) {
            sb.append("Request time [");
            sb.append(i);
            sb.append("] : ");
            sb.append(requestTime[i]);
            sb.append(" ms\n");
                        
            averageTime += requestTime[i];
        }
                        
        sb.append("Average call request time : ");
        sb.append(averageTime / intervals.length);
        sb.append(" ms\n");
        syslog.log(sb);
                
                
        // Gets the result.
        // This is not optimal: use group instead !!
        PiComputer.Result[] results = new PiComputer.Result[intervals.length];
                
        for (int i = 0; i < intervals.length; i++) {
            try{
                results[i] = (PiComputer.Result) futures[i].waitForResult();
            }catch(Throwable t) {
                t.printStackTrace();
                System.exit(1);
            }   
        }       
                
        // Compute average results.computedTime and place it in results[0] !
        sb = new StringBuffer(500);
        sb.append("Computed time summary \n");

        long computedTime = 0;
        java.math.BigDecimal bd = new java.math.BigDecimal(0);
        
        for (int i = 0; i < intervals.length; i++) {
            sb.append("Computed time [");
            sb.append(i);
            sb.append("] : ");
            sb.append(results[i].computedTime);
            sb.append(" ms\n");
            computedTime += results[i].computedTime;
            bd = bd.add(results[i].bd);
        }

        sb.append("Average call computed time : ");
        sb.append(computedTime / intervals.length);
        sb.append(" ms\n");
        syslog.log(sb);
                
        PiComputer.logResult(new IntInterval(0, intervals[intervals.length - 1].end), 
                             intervals.length,
                             new PiComputer.Result(bd, computedTime),
                             (System.currentTimeMillis() - startChrono));
    }
}

