package pi;

import mandala.jacob.*;
import mandala.rami.*;
import mandala.rami.impl.*;
import mandala.util.*;

public class TestPerfRemotePiComputer   {
    public static Syslog syslog = TestPerfPiComputer.syslog;
        
    public static void main(String[] args) {
        if (args.length != 4) {
            syslog.error(PiComputer.usage(TestPerfRemotePiComputer.class, 
                                          "times RemoteActiveMapAddress"));
            System.exit(1);
        }
        
        long startChrono = System.currentTimeMillis();
                
        int scale = Integer.parseInt(args[0]);
        IntInterval[] intervals = 
            new IntInterval[Integer.parseInt(args[2]) + 1];
        PiComputer.distribute(intervals, Integer.parseInt(args[0]));
        
        // gets a proxy on a remote active map
        ActiveMap activeMap = null;
        try{
            activeMap = (mandala.jacob.remote.RemoteActiveMap)
                new javax.naming.InitialContext().lookup(args[3]);
        }catch(Exception e) {
            e.printStackTrace();
            System.exit(1);
        }
        
        // Constructs the factory which generates RSOR(computer)
        
        Framework.Factory RSORFactory = new SORFactory(activeMap);
        
        
        // Then this factory generates RSOR()
        Framework.setFactory(RSORFactory);
        
        jaya.pi.PiComputer proxy;
        
        for (int i = 0; i < 10; i++) {
            // Dummy call to prevent caches effects !
            proxy = new jaya.pi.PiComputer(scale);
            try{
                proxy.rami_compute(new IntInterval(10, 20), 
                                   args[1]).waitForResult();
            }catch(Throwable t) {
                syslog.error(t);
                System.exit(1);
            }
        }
        
        System.gc();
                
        // We must insert a new PiComputer each time since
        // StoredObjectReference implements a singleton mechanism !!
        syslog.log("-- Using RSOR(PiComputer) --\n");
        Framework.Factory factory = RSORFactory, 
            headFactory = RSORFactory, 
            tailFactory = RSORFactory;
        
        remoteCompute(factory, 
                      scale,
                      intervals, 
                      args[1]);
        
        syslog.log("-- Using ARP(AR_Sync(RSOR(PiComputer))) --\n");
        
	AsynchronousPolicyFactory synchronousAPF = 
	    new AsynchronousPolicyFactory() {
		public AsynchronousPolicy getAsynchronousPolicy(Object o) { 
		    return new SynchronousSemantic();
		}
	    };

	headFactory = new ARFactory(synchronousAPF);
        factory = new ARPFactory(headFactory, tailFactory);
        remoteCompute(factory,
                      scale, 
                      intervals,
                      args[1]);
                
        syslog.log("-- Using ARP(AR_Threaded(RSOR(PiComputer))) --\n");
        ((ARFactory) headFactory).setDefaultAPFactory(new ThreadedAPFactory());
        remoteCompute(factory, 
                      scale, 
                      intervals,
                      args[1]);
	
        syslog.log("-- Using ARP(AR_ThreadPooled(RSOR(PiComputer))) --\n");
        ((ARFactory) headFactory)
	    .setDefaultAPFactory(new ThreadPooledAPFactory());
        remoteCompute(factory, 
                      scale, 
                      intervals,
                      args[1]);
        
        syslog.log("-- Using ARP(RSOR(AR_Sync(PiComputer))) --\n");
        FutureFactory futureFactory = 
            ((mandala.jacob.remote.RemoteActiveMap) 
             activeMap)
            .getDefaultFutureFactoryWrapper(ARFactory
					    .getDefaultFutureFactory());
        tailFactory = new ARFactory(futureFactory, synchronousAPF);
        headFactory = RSORFactory;
        factory = new ARPFactory(headFactory, tailFactory);
        remoteCompute(factory, 
                      scale, 
                      intervals,
                      args[1]);
        
        syslog.log("-- Using ARP(RSOR(AR_Threaded(PiComputer))) --\n");
        ((ARFactory) tailFactory).setDefaultAPFactory(new ThreadedAPFactory());
        factory = new ARPFactory(headFactory, tailFactory);
        remoteCompute(factory, 
                      scale, 
                      intervals,
                      args[1]);
        
        syslog.log("-- Using ARP(RSOR(AR_ThreadPooled(PiComputer))) --\n");
        ((ARFactory) tailFactory)
	    .setDefaultAPFactory(new ThreadPooledAPFactory());
        factory = new ARPFactory(headFactory, tailFactory);
        remoteCompute(factory, 
                      scale, 
                      intervals,
                      args[1]);
        
        
        syslog.log("Everything done in " + 
                   (System.currentTimeMillis() - startChrono) + "ms");
    }
    
    public static void remoteCompute(Framework.Factory factory,
                                     int scale,
                                     IntInterval[] intervals,
                                     String strategy) {
	jaya.pi.PiComputer proxy = new jaya.pi.PiComputer(scale, factory);
        TestPerfPiComputer.compute(proxy, intervals, strategy);
    }
}

