package pi;

import java.math.*;

import mandala.rami.*;
import mandala.util.*;

public class TotalTransparentAsynchronousPiComputer 
    implements java.io.Serializable {
    public static Syslog syslog = new Syslog();

    // Interface declaration and implementation for the total asynchronous
    // method invocation mechanism
    public interface PiComputerOp {
        String RAW = PiComputer.RAW;
        String SAFE = PiComputer.SAFE;
        String SAFE_PERF = PiComputer.SAFE_PERF;
                
        PiComputerOp.ResultOp compute(IntInterval interval, String method);
                
        interface ResultOp {
            BigDecimal getBigDecimal();
            long getComputedTime();
            PiComputer.Result getRealResult();
        }
    }
        
    public static class PiComputerImpl implements PiComputerOp{
        private PiComputer computer;
                
        public PiComputerImpl(int scale) {
            computer = new PiComputer(scale);
        }
                
        public PiComputerOp.ResultOp compute(IntInterval interval, String method) {
            return new ResultImpl(computer.compute(interval, method));
        }
                
        public static class ResultImpl implements PiComputerOp.ResultOp {
            PiComputer.Result result;
            public ResultImpl(PiComputer.Result result) {
                this.result = result;
            }

            public PiComputer.Result getRealResult() {
                return result;
            }
                        
            public BigDecimal getBigDecimal() {
                return result.bd;
            }
                        
            public long getComputedTime() {
                return result.computedTime;
            }
        }
    }

    public static final String USAGE = 
        "Usage : java " + 
        TotalTransparentAsynchronousPiComputer.class.getName() + 
        " Scale StrategyCode\n" + 
        "Where StrategyCode can be : \n\"raw\", \"safe\", or \"safePerf\"";
        
    public static void main(String[] args) {
        if (args.length != 2) {
            syslog.error(USAGE);
            System.exit(1);
        }
                
        long startChrono = System.currentTimeMillis();
                
        IntInterval interval = new IntInterval(0,
                                               Integer.parseInt(args[0]));
                

        PiComputerOp computer = new PiComputerImpl(interval.end);
                
        PiComputerOp proxy = (PiComputerOp) 
            Framework.getTotalTransparentAsynchronousProxy(computer);
        
        // Total transparent asynchronous call !!
        PiComputerOp.ResultOp result = 
            (PiComputerOp.ResultOp) proxy.compute(interval, args[1]);
        
        // Something can be done here during the previous call execution !
                
        // computeSomeThing();
        
        // Then, the wait by necessity mechanism is used here :
        // the current thread is blocked if the result is unavailable !
        PiComputer.logResult(interval,
                             1,
                             result.getRealResult(),
                             System.currentTimeMillis() - startChrono);
    }
}   
